#include "power_buffer.h"

#include <string.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "fft.h"

void power_buffer_init(power_buffer *b, int width, int height,
                     void (*bnl)(void *), void *bnl_data)
{
  b->buffer_new_line = bnl;
  b->bnl_data = bnl_data;
  b->width = width;
  b->power = malloc(sizeof(float) * width * 2049);
  b->buffer = malloc(sizeof(double) * 4096);
  b->fft_buffer = malloc(sizeof(complex) * 4096);
  if (!b->power || !b->buffer || !b->fft_buffer) { printf("oom\n"); exit(1); }
  memset(b->power, 0, sizeof(float) * width * 2049);
  b->processed = 0;
  if (open_fft(&b->fft, 4096)) { printf("err open fft\n"); exit(1); }
}

static void compute_power(power_buffer *b)
{
  int i;
  fft_double(&b->fft, b->buffer, b->fft_buffer);
  for (i = 0; i < 2049; i++)
    memmove(&b->power[b->width * i], &b->power[b->width * i + 1],
            (b->width - 1) * sizeof(float));
  for (i = 0; i < 2049; i++) {
    b->power[b->width * i + b->width-1] =
        sqrt(b->fft_buffer[i].re * b->fft_buffer[i].re +
             b->fft_buffer[i].im * b->fft_buffer[i].im); // / 2048; //4096;
  }
}

void power_buffer_feed(power_buffer *b, float *buf, int len)
{
  int i;
  for (i = 0; i < len; i++) {
    if (b->processed == 4096) {
      b->processed = 0;
      compute_power(b);
      b->buffer_new_line(b->bnl_data);
    }
    b->buffer[b->processed] = buf[i];
    b->processed++;
  }
}
